/*{
	"CREDIT": "Jason Beyers",
	"DESCRIPTION": "Routes a single input signal to one of 8 possible outputs.",
	"TAGS": "data",
	"VSN": "1.0",
	"INPUTS": [
		{
			"LABEL": "Input/Input",
			"NAME": "input",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"DESCRIPTION": "Input"
		},
		{
			"LABEL": "Settings/Update On Selection Change",
			"NAME": "switch_on_mode_change",
			"TYPE": "bool",
			"DEFAULT": "true",
			"FLAGS" : "button"
		},
		{
			"LABEL": "Settings/Set Inactive Outputs to Value",
			"NAME": "set_inactive_outputs_to_value",
			"TYPE": "bool",
			"DEFAULT": "false",
			"FLAGS" : "button",
			"DESCRIPTION": "Flag to use a preset value for inactive outputs during an output switch"
		},
		{
			"LABEL": "Settings/Value",
			"NAME": "inactive_value",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "hidden",
			"DESCRIPTION": "Preset value to apply to inactive outputs (once) during an output switch"
		},
		{
			"LABEL": "Output/Range",
			"NAME": "output_range",
			"TYPE": "floatRange",
			"DEFAULT": [0.0,1.0],
			"MIN": 0.0,
			"MAX": 1.0
		},
		{
			"LABEL": "Output/Select",
			"NAME": "mode",
			"TYPE": "long",
			"DEFAULT": "1",
			"VALUES": ["1", "2", "3", "4", "5", "6", "7", "8"],
			"DESCRIPTION": "Select output"
		}
	],
	"OUTPUTS": [
		{
			"LABEL": "Output/Output 1",
			"NAME": "output1",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		},
		{
			"LABEL": "Output/Output 2",
			"NAME": "output2",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		},
		{
			"LABEL": "Output/Output 3",
			"NAME": "output3",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		},
		{
			"LABEL": "Output/Output 4",
			"NAME": "output4",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		},
		{
			"LABEL": "Output/Output 5",
			"NAME": "output5",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		},
		{
			"LABEL": "Output/Output 6",
			"NAME": "output6",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		},
		{
			"LABEL": "Output/Output 7",
			"NAME": "output7",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		},
		{
			"LABEL": "Output/Output 8",
			"NAME": "output8",
			"TYPE": "float",
			"DEFAULT": 0,
			"MIN": 0,
			"MAX": 1,
			"FLAGS": "read_only"
		}
	]
}*/

function init()
	-- Add listeners
	lastInputChange = 0
	mad.listen("/self/input", "onInputChanged")
	mad.listen("/self/switch_on_mode_change", "onSwitchChanged")
	mad.listen("/self/set_inactive_outputs_to_value", "onInactivePresetFlagChanged")
	-- mad.listen("/self/inactive_value", "onInactivePresetChanged")
	mad.listen("/self/mode", "onModeChanged")
	mad.listen("/self/output_range", "updateOutput")
	onModeChanged()
	-- updateOutput()
end

function updateOutput()
	local settings = mad.get_attr("/self")
	if settings["active"] == false then
		return
	end

	local mode = settings["mode"]
	local inputValue = settings["input"]

	-- Apply output range restrictions
	local min = settings["output_range"][1]
	local max = settings["output_range"][2]
	if inputValue < 0 then inputValue = 0 end
	if inputValue > 1 then inputValue = 1 end

	-- Pass the input to only one of the outputs
	if mode == "1" then
		mad.set("/self/output1",min + inputValue * (max-min))
	elseif mode == "2" then
		mad.set("/self/output2",min + inputValue * (max-min))
	elseif mode == "3" then
		mad.set("/self/output3",min + inputValue * (max-min))
	elseif mode == "4" then
		mad.set("/self/output4",min + inputValue * (max-min))
	elseif mode == "5" then
		mad.set("/self/output5",min + inputValue * (max-min))
	elseif mode == "6" then
		mad.set("/self/output6",min + inputValue * (max-min))
	elseif mode == "7" then
		mad.set("/self/output7",min + inputValue * (max-min))
	else
		mad.set("/self/output8",min + inputValue * (max-min))
	end
end

function updateOutputWithPresets()
	local settings = mad.get_attr("/self")
	if settings["active"] == false then
		return
	end

	local mode = settings["mode"]
	local inputValue = settings["input"]

	local presetValue = settings["inactive_value"]

	-- Apply output range restrictions
	local min = settings["output_range"][1]
	local max = settings["output_range"][2]
	if inputValue < 0 then inputValue = 0 end
	if inputValue > 1 then inputValue = 1 end

	if presetValue < 0 then presetValue = 0 end
	if presetValue > 1 then presetValue = 1 end

	-- Pass the input to only one of the outputs
	if mode == "1" then
		mad.set("/self/output1",min + inputValue * (max-min))
		-- mad.set("/self/output1",min + presetValue * (max-min))
		mad.set("/self/output2",min + presetValue * (max-min))
		mad.set("/self/output3",min + presetValue * (max-min))
		mad.set("/self/output4",min + presetValue * (max-min))
		mad.set("/self/output5",min + presetValue * (max-min))
		mad.set("/self/output6",min + presetValue * (max-min))
		mad.set("/self/output7",min + presetValue * (max-min))
		mad.set("/self/output8",min + presetValue * (max-min))
	elseif mode == "2" then
		mad.set("/self/output2",min + inputValue * (max-min))
		mad.set("/self/output1",min + presetValue * (max-min))
		-- mad.set("/self/output2",min + presetValue * (max-min))
		mad.set("/self/output3",min + presetValue * (max-min))
		mad.set("/self/output4",min + presetValue * (max-min))
		mad.set("/self/output5",min + presetValue * (max-min))
		mad.set("/self/output6",min + presetValue * (max-min))
		mad.set("/self/output7",min + presetValue * (max-min))
		mad.set("/self/output8",min + presetValue * (max-min))
	elseif mode == "3" then
		mad.set("/self/output3",min + inputValue * (max-min))
		mad.set("/self/output1",min + presetValue * (max-min))
		mad.set("/self/output2",min + presetValue * (max-min))
		-- mad.set("/self/output3",min + presetValue * (max-min))
		mad.set("/self/output4",min + presetValue * (max-min))
		mad.set("/self/output5",min + presetValue * (max-min))
		mad.set("/self/output6",min + presetValue * (max-min))
		mad.set("/self/output7",min + presetValue * (max-min))
		mad.set("/self/output8",min + presetValue * (max-min))
	elseif mode == "4" then
		mad.set("/self/output4",min + inputValue * (max-min))
		mad.set("/self/output1",min + presetValue * (max-min))
		mad.set("/self/output2",min + presetValue * (max-min))
		mad.set("/self/output3",min + presetValue * (max-min))
		-- mad.set("/self/output4",min + presetValue * (max-min))
		mad.set("/self/output5",min + presetValue * (max-min))
		mad.set("/self/output6",min + presetValue * (max-min))
		mad.set("/self/output7",min + presetValue * (max-min))
		mad.set("/self/output8",min + presetValue * (max-min))
	elseif mode == "5" then
		mad.set("/self/output5",min + inputValue * (max-min))
		mad.set("/self/output1",min + presetValue * (max-min))
		mad.set("/self/output2",min + presetValue * (max-min))
		mad.set("/self/output3",min + presetValue * (max-min))
		mad.set("/self/output4",min + presetValue * (max-min))
		-- mad.set("/self/output5",min + presetValue * (max-min))
		mad.set("/self/output6",min + presetValue * (max-min))
		mad.set("/self/output7",min + presetValue * (max-min))
		mad.set("/self/output8",min + presetValue * (max-min))
	elseif mode == "6" then
		mad.set("/self/output6",min + inputValue * (max-min))
		mad.set("/self/output1",min + presetValue * (max-min))
		mad.set("/self/output2",min + presetValue * (max-min))
		mad.set("/self/output3",min + presetValue * (max-min))
		mad.set("/self/output4",min + presetValue * (max-min))
		mad.set("/self/output5",min + presetValue * (max-min))
		-- mad.set("/self/output6",min + presetValue * (max-min))
		mad.set("/self/output7",min + presetValue * (max-min))
		mad.set("/self/output8",min + presetValue * (max-min))
	elseif mode == "7" then
		mad.set("/self/output7",min + inputValue * (max-min))
		mad.set("/self/output1",min + presetValue * (max-min))
		mad.set("/self/output2",min + presetValue * (max-min))
		mad.set("/self/output3",min + presetValue * (max-min))
		mad.set("/self/output4",min + presetValue * (max-min))
		mad.set("/self/output5",min + presetValue * (max-min))
		mad.set("/self/output6",min + presetValue * (max-min))
		-- mad.set("/self/output7",min + presetValue * (max-min))
		mad.set("/self/output8",min + presetValue * (max-min))
	else
		mad.set("/self/output8",min + inputValue * (max-min))
		mad.set("/self/output1",min + presetValue * (max-min))
		mad.set("/self/output2",min + presetValue * (max-min))
		mad.set("/self/output3",min + presetValue * (max-min))
		mad.set("/self/output4",min + presetValue * (max-min))
		mad.set("/self/output5",min + presetValue * (max-min))
		mad.set("/self/output6",min + presetValue * (max-min))
		mad.set("/self/output7",min + presetValue * (max-min))
		-- mad.set("/self/output8",min + presetValue * (max-min))
	end

end

function onInputChanged(event)
	lastInputChange = event.time
	updateOutput()
end

function onModeChanged(event)

	local settings = mad.get_attr("/self")
	if settings["active"] == false then
		return
	end

	local switch_on_mode_change = settings["switch_on_mode_change"]
	local set_inactive_outputs_to_value = settings["set_inactive_outputs_to_value"]

	if switch_on_mode_change then
		if set_inactive_outputs_to_value then
			updateOutputWithPresets()
		else
			updateOutput()
		end
	end
end

function onSwitchChanged(event)

	local settings = mad.get_attr("/self")
	if settings["active"] == false then
		return
	end

	local switch_on_mode_change = settings["switch_on_mode_change"]

	if switch_on_mode_change then
		mad.set_attr_settings("/self/set_inactive_outputs_to_value",{flags_to_remove="hidden"})
	else
		mad.set_attr_settings("/self/set_inactive_outputs_to_value",{flags_to_add="hidden"})
		mad.set_attr_settings("/self/inactive_value",{flags_to_add="hidden"})
	end

	-- updateOutput()
end


function onInactivePresetFlagChanged(event)

	local settings = mad.get_attr("/self")
	if settings["active"] == false then
		return
	end

	local set_inactive_outputs_to_value = settings["set_inactive_outputs_to_value"]

	if set_inactive_outputs_to_value then
		mad.set_attr_settings("/self/inactive_value",{flags_to_remove="hidden"})
	else
		mad.set_attr_settings("/self/inactive_value",{flags_to_add="hidden"})
	end

	-- updateOutput()
end